@@ -455,6 +455,9 @@ class LessNode(Node):
 
 @NODE_REGISTRY.register()
 class LessOrEqualNode(LessNode):
+    def shape_infer(self, intensors: []):
+        return [_get_shape(intensors[0])]
+
     def value_infer(self, intensors: [numpy.ndarray]):
         result = numpy.less_equal(intensors[0], intensors[1])
         return [result]
@@ -476,6 +479,9 @@ class AndNode(Node):
 
 @NODE_REGISTRY.register()
 class WhereNode(Node):
+    def shape_infer(self, intensors: []):
+        return [_get_shape(intensors[0])]  # return condition shape
+
     def value_infer(self, intensors: []):
         result = numpy.where(intensors[0], intensors[1], intensors[2])
         return [result]
@@ -491,7 +497,7 @@ class TransposeNode(Node):
         xshape = _get_shape(intensors[0])
         yshape = []
         if self.perm is None:
-            return [xshape]
+            return [xshape[::-1]]
         for axis in self.perm:
             yshape.append(xshape[axis])
         return [yshape]
@@ -684,7 +690,11 @@ class UnsqueezeNode(Node):
 
     def shape_infer(self, intensors: []):
         inshape = _get_shape(intensors[0])
-        axes = _axes_neg2pos(len(inshape), self.axes)
+        if len(intensors) == 2:
+            axes = intensors[1]
+        else:
+            axes = self.axes
+        axes = _axes_neg2pos(len(inshape), axes)
         newshape = []
         idx = 0
         for i in range(len(inshape) + len(axes)):
@@ -697,7 +707,7 @@ class UnsqueezeNode(Node):
 
     def value_infer(self, intensors: []):
         outtensor = intensors[0]
-        if self.axes is None:
+        if len(intensors) == 2:
             axes = intensors[1]
         else:
             axes = self.axes
@@ -1107,13 +1117,17 @@ class ReduceMeanNode(Node):
     def shape_infer(self, intensors: []):
         xshape = _get_shape(intensors[0])
         yshape = []
-        if self.axes is None:
+        if len(intensors) == 2:
+            axes = intensors[1]
+        else:
+            axes = self.axes
+        if axes is None:
             return [[1]]
         else:
-            self.axes = _axes_neg2pos(len(xshape), self.axes)
+            axes = _axes_neg2pos(len(xshape), axes)
 
         for i in range(len(xshape)):
-            if i in self.axes:
+            if i in axes:
                 if self.keepdims:
                     yshape.append(1)
             else:
@@ -1121,7 +1135,11 @@ class ReduceMeanNode(Node):
         return [yshape]
 
     def value_infer(self, intensors: [numpy.ndarray]):
-        reduced = numpy.mean(intensors[0], axis=self.axes, keepdims=self.keepdims == 1)
+        if len(intensors) == 2:
+            axes = intensors[1]
+        else:
+            axes = self.axes
+        reduced = numpy.mean(intensors[0], axis=axes, keepdims=self.keepdims == 1)
         return [reduced]
 
     def profile(self, intensors: [numpy.ndarray], outtensors: [numpy.ndarray]):
@@ -1132,7 +1150,11 @@ class ReduceMeanNode(Node):
 @NODE_REGISTRY.register()
 class ReduceProdNode(ReduceMeanNode):
     def value_infer(self, intensors: [numpy.ndarray]):
-        reduced = numpy.prod(intensors[0], axis=self.axes, keepdims=self.keepdims == 1)
+        if len(intensors) == 2:
+            axes = intensors[1]
+        else:
+            axes = self.axes
+        reduced = numpy.prod(intensors[0], axis=axes, keepdims=self.keepdims == 1)
         return [reduced]
 
     def profile(self, intensors: [numpy.ndarray], outtensors: [numpy.ndarray]):
@@ -1144,7 +1166,11 @@ class ReduceProdNode(ReduceMeanNode):
 @NODE_REGISTRY.register()
 class ReduceSumNode(ReduceMeanNode):
     def value_infer(self, intensors: [numpy.ndarray]):
-        reduced = numpy.sum(intensors[0], axis=self.axes, keepdims=self.keepdims == 1)
+        if len(intensors) == 2:
+            axes = intensors[1]
+        else:
+            axes = self.axes
+        reduced = numpy.sum(intensors[0], axis=axes, keepdims=self.keepdims == 1)
         return [reduced]
 
 
@@ -1152,7 +1178,11 @@ class ReduceSumNode(ReduceMeanNode):
 class ReduceMinNode(ReduceMeanNode):
     def value_infer(self, intensors: [numpy.ndarray]):
         data = intensors[0]
-        reduced = numpy.minimum.reduce(data, axis=self.axes, keepdims=self.keepdims == 1)
+        if len(intensors) == 2:
+            axes = intensors[1]
+        else:
+            axes = self.axes
+        reduced = numpy.minimum.reduce(data, axis=axes, keepdims=self.keepdims == 1)
         return [reduced]
 
     def profile(self, intensors: [numpy.ndarray], outtensors: [numpy.ndarray]):
@@ -1165,7 +1195,11 @@ class ReduceMinNode(ReduceMeanNode):
 class ReduceMaxNode(ReduceMinNode):
     def value_infer(self, intensors: [numpy.ndarray]):
         data = intensors[0]
-        reduced = numpy.maximum.reduce(data, axis=self.axes, keepdims=self.keepdims == 1)
+        if len(intensors) == 2:
+            axes = intensors[1]
+        else:
+            axes = self.axes
+        reduced = numpy.maximum.reduce(data, axis=axes, keepdims=self.keepdims == 1)
         return [reduced]
 
 
@@ -1620,6 +1654,9 @@ class ConstantOfShapeNode(Node):
         super().__init__(nodeproto)
         self.add_default_value('value', None)
 
+    def shape_infer(self, intensors: []):
+        return [list(intensors[0].astype(numpy.int64))]
+
     def value_infer(self, intensors: []):
         arr = numpy.zeros(intensors[0].astype(numpy.int64), dtype=self.value.dtype)
         if self.value is not None and len(self.value) == 1:
@@ -1643,22 +1680,51 @@ class SplitNode(Node):
         self.add_default_value('axis', None)
         self.add_default_value('split', None)
 
-    def value_infer(self, intensors: []):
-        split = []
+    def shape_infer(self, intensors: []):
         end = 0
+        inshape = _get_shape(intensors[0])
         if self.split is None:
             if len(intensors) == 2:
-                self.split = intensors[1]
+                split = intensors[1]
             else:
-                self.split = [intensors[0].shape[self.axis] // 2]
-
-        self.axis = _axes_neg2pos(len(intensors[0].shape), [self.axis])[0]
-        for v in self.split:
-            if end + v >= intensors[0].shape[self.axis]:
-                break
-            split.append(end + v)
+                split = [inshape[self.axis] // 2] * 2
+        else:
+            split = self.split
+        axis = _axes_neg2pos(len(inshape), [self.axis])[0]
+        shapes = []
+        for v in split:
+            shape = []
+            for i in range(len(inshape)):
+                if i == axis:
+                    if end + v < inshape[axis]:
+                        shape.append(v)
+                    else:
+                        shape.append(inshape[axis] - end)
+                else:
+                    shape.append(inshape[i])
             end += v
-        return numpy.split(intensors[0], split, self.axis)
+            shapes.append(shape)
+        return shapes
+
+    def value_infer(self, intensors: []):
+        splitpos = []
+        end = 0
+        inshape = _get_shape(intensors[0])
+        if self.split is None:
+            if len(intensors) == 2:
+                split = intensors[1]
+            else:
+                split = [inshape[self.axis] // 2]
+        else:
+            split = self.split
+
+        axis = _axes_neg2pos(len(inshape), [self.axis])[0]
+        for v in split:
+            if end + v >= inshape[axis]:
+                break
+            splitpos.append(end + v)
+            end += v
+        return numpy.split(intensors[0], splitpos, axis)
 
 def create_node(n: onnx.NodeProto):
     node_class = NODE_REGISTRY.get(n.op_type + 'Node')
